#!/usr/bin/env python
#
#   This file is part of IntiPunku
#   Copyright (C) 2007 Mark Baas <mark.baas123@gmail.com>
#
#   This program is free software: you can redistribute it and/or modify
#   it under the terms of the GNU Affero General Public License as
#   published by the Free Software Foundation, either version 3 of the
#   License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU Affero General Public License for more details.
#
#   You should have received a copy of the GNU Affero General Public License
#   along with this program.  If not, see <http://www.gnu.org/licenses/>.

import gtk, gc, os.path, os, gobject, cairo
from inti import tools
from inti.constants import *

from distutils import dir_util, file_util
import StringIO
from inti import utils
import gc, time
from inti import dialogs
from inti import metadata
import pango

import gtkimageview

# Localization
import gettext
_ = gettext.gettext

DATADIR = os.environ["INTIDATA"]

# Tool page constants
PAGE_CROP = 1
PAGE_REDEYES = 2
PAGE_BLUESKY = 3

class SunEditor(gtk.HPaned):
    __gsignals__ = {
        'image-changed' : (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE, (gobject.TYPE_STRING,)),
        }

    def __init__(self, win, callback, minibrowser=None):
        gtk.HPaned.__init__(self)

        gobject.GObject.__init__(self)

        self.set_position(210)

        self.win = win
        self.callback = callback
        self.minibrowser = minibrowser


        # Setup the Draw widget      
        self.draw = SunDraw(self, self.callback)
        self.draw.connect('scroll-event', self.scroll_event)
        self.win.connect('key-press-event', self.key_press_event)
        self.win.connect('key-release-event', self.key_release_event)
        self.allow_scroll = True
        
        """
        LEFT SIDE 
        """

        self.notebook = gtk.Notebook()
        self.notebook.set_show_tabs(False)
        vbox = gtk.VBox()
        vbox.set_spacing(5)
        vbox.set_border_width(10)

        # Back button
        button = gtk.Button(stock=gtk.STOCK_GO_BACK)
        button.connect("clicked", self.sunset)
        vbox.pack_start(button, expand=False, fill=False)

        ###
        # Visible notebook voor all enhancements
        # tab 1: adjustments: all controls and auto-adjustments (equalize/autocontrast & plugins)
        # tab 2: tools: crop, red eye, etc
        # tab 3: effects: todo

        self.notebookv = gtk.Notebook()

        # TAB 1: Adjustments
        vbox_tab1 = gtk.VBox()
        vbox_tab1.set_border_width(5)
        self.notebookv.append_page(vbox_tab1,tab_label=gtk.Label(_("Adjustments")))
        
        vbox_expan = gtk.VBox()
        vbox_expan.set_border_width(5)
        expan = gtk.Expander(label=_("Advanced controls"))
        expan.add(vbox_expan)
        vbox_tab1.pack_end(expan, fill=False, expand=False)
        
        self.controls = tools.Controls()
        
        # Add plugins to Control capabilities
        pluginmgr = Plugins()
        plugins = pluginmgr.get_list()
        for plugin in plugins:
            self.controls.add_capability(*plugin)

        controls = self.controls.get_controls()
        self.scales = []
        for control, name, desc in controls:
            hbox = gtk.HBox()
            label = gtk.Label(str=name)
            vbox_expan.pack_start(label, expand=False, fill=False)

            plbutton = gtk.Button()
            plbutton.set_relief(gtk.RELIEF_NONE)
            gim = gtk.Image()
            gim.set_from_pixbuf(utils.load_icon("add", 16))#.scale_simple(8,8,gtk.gdk.INTERP_BILINEAR))
            plbutton.add(gim)

            mibutton = gtk.Button()
            mibutton.set_relief(gtk.RELIEF_NONE)
            gim = gtk.Image()
            gim.set_from_pixbuf(utils.load_icon("remove", 16))#.scale_simple(8,8,gtk.gdk.INTERP_BILINEAR))
            mibutton.add(gim)

            scale = gtk.HScale()
            scale.set_range(0,200)
            scale.set_draw_value(False)
            scale.set_value(100)
            scale.set_name(control)
            scale.set_tooltip_text(desc)
            scale.connect("value-changed", self.handle_draw_movement)
            scale.connect("button-press-event", self.handle_draw_prep)
            scale.connect("button-release-event", self.handle_draw_do)

            plbutton.connect("clicked", self.handle_scale, 1, scale)
            mibutton.connect("clicked", self.handle_scale, -1, scale)

            hbox.pack_start(mibutton, expand=False, fill=False)
            hbox.pack_start(scale, expand=True, fill=True)
            hbox.pack_start(plbutton, expand=False, fill=False)
            vbox_expan.pack_start(hbox, expand=False, fill=False)
            self.scales.append(scale)

        # auto tools
        vbox_autotool = gtk.VBox()
        scroll_autotool = gtk.ScrolledWindow()
        scroll_autotool.add_with_viewport(vbox_autotool)
        scroll_autotool.set_policy(gtk.POLICY_NEVER, gtk.POLICY_AUTOMATIC)
        vbox_tab1.pack_start(scroll_autotool)
        autocontrols = self.controls.get_autotools()
        for control, name, desc in autocontrols:
            button = gtk.Button()
            button.set_tooltip_text(desc)
            button.set_label(name)
            button.set_relief(gtk.RELIEF_NONE)
            button.connect("clicked", self.handle_draw_auto)
            button.set_name(control)
            bim = gtk.image_new_from_pixbuf(utils.load_icon(control))
            button.set_image(bim)
            align = gtk.Alignment()
            align.add(button)
            vbox_autotool.pack_start(align, expand=False, fill=False)
            

        # TAB 2: Tools
        vbox_tab2 = gtk.VBox()
        vbox_tab2.set_border_width(5)
        self.notebookv.append_page(vbox_tab2,tab_label=gtk.Label(_("Tools")))
        buttons = {_("Crop"):"crop",
                   _("Red eyes"):"redeyes",
                   _("Blue sky"): "bluesky",
                  }
        for button in buttons:
            b = gtk.Button(label=_(button))
            b.set_name(buttons[button])
            b.connect("clicked", self.handle_buttons)
            bim = gtk.image_new_from_pixbuf(utils.load_icon(buttons[button]))
            b.set_image(bim)
            vbox_tab2.pack_start(b, expand=False, fill=False)
        
        vbox.pack_start(self.notebookv)

        # Notebook end
        ###

        # Undo button
        reset_button = gtk.Button(label=_("Undo all"))
        reset_button.set_image(gtk.image_new_from_pixbuf(utils.load_icon("stock_undo")))
        reset_button.set_name("Reset")
        reset_button.connect("clicked", self.handle_draw_reset)
        vbox.pack_start(reset_button, fill=False, expand=False)

        # Save button
        save_button = gtk.Button(label=_("Save now"))
        save_button.set_image(gtk.image_new_from_pixbuf(utils.load_icon("stock_save")))
        save_button.set_name("Save")
        save_button.connect("clicked", self.save_changes)
        vbox.pack_start(save_button, fill=False, expand=False)

        # Save button2
        save_button2 = gtk.Button(label=_("Save to new file"))
        save_button2.set_image(gtk.image_new_from_pixbuf(utils.load_icon("stock_save")))
        save_button2.set_name("Savenew")
        save_button2.connect("clicked", self.save_changes, True)
        vbox.pack_start(save_button2, fill=False, expand=False)
      
        # - Revert to orignal - button
        self.buttonorig = gtk.Button()
        self.buttonorig.set_label(_("Revert to original"))
        self.buttonorig.set_image(gtk.image_new_from_pixbuf(utils.load_icon("revert")))
        self.buttonorig.connect("clicked", self.revert_to_orig)
        vbox.pack_end(self.buttonorig, expand=False, fill=False)

        """
        RIGHT SIDE
        """
        vbox2 = gtk.VBox()
        vbox2.set_border_width(5)
        vbox2.set_spacing(5)

        # FOOTER OF DRAW
        hbox_footer = gtk.HBox()
        # rotat
        frame_rotate = gtk.Frame(label=_("Rotation"))
        hbox_rotate = gtk.HBox()
        rrb = gtk.Button(label=_("Right"))
        rrb.set_image(gtk.image_new_from_pixbuf(utils.load_icon("rotate-right")))
        rrb.connect("clicked", self.rotate, 1)
        hbox_rotate.pack_end(rrb, fill=False, expand=False)
        rlb = gtk.Button(label=_("Left"))
        rlb.set_image(gtk.image_new_from_pixbuf(utils.load_icon("rotate-left")))
        rlb.connect("clicked", self.rotate, 0)
        hbox_rotate.pack_start(rlb, fill=False, expand=False)   
        frame_rotate.add(hbox_rotate)
        hbox_footer.pack_start(frame_rotate, fill=False, expand=False)

        # Desc label
        vbox1 = gtk.VBox()
        self.desc_label = gtk.Label()   
        self.desc_label.set_ellipsize(pango.ELLIPSIZE_MIDDLE)
        self.desc_label.set_size_request(350,30)
        vbox1.pack_start(self.desc_label)
        hbox_footer.pack_start(vbox1, fill=True, expand=True)

        # Zoom buttons
        frame_zoom = gtk.Frame(label=_("Zoom"))
        zoomin_b = gtk.Button(_("Zoom in"))
        zoomin_b.set_image(gtk.image_new_from_pixbuf(utils.load_icon("stock_zoom-in")))
        zoomout_b = gtk.Button(_("Zoom out"))
        zoomout_b.set_image(gtk.image_new_from_pixbuf(utils.load_icon("stock_zoom-out")))
        zoomin_b.connect("clicked", self.zoom, 0)
        zoomout_b.connect("clicked", self.zoom, 1)
        zoom_hb = gtk.HBox()
        zoom_hb.pack_start(zoomin_b, fill=False, expand=True)
        zoom_hb.pack_end(zoomout_b, fill=False, expand=True)
        frame_zoom.add(zoom_hb)
        hbox_footer.pack_end(frame_zoom, fill=False, expand=False)
        
        vbox2.pack_end(hbox_footer, fill=False, expand=False)
        
        # Minibrowser
        if self.minibrowser:
            align = gtk.Alignment(xalign=0.5)
            align.add(self.minibrowser)
            vbox2.pack_start(align, fill=False, expand=False)

        # Add draw widget
        scroll = gtkimageview.ImageScrollWin(self.draw)
        vbox2.pack_start(scroll)


        # Tool pages    
        
        vboxtools = (ToolPage(_("Crop selection"),"crop",_("Select the area you would like to crop"),self.cb_tools),
                     ToolPage(_("Red eyes"),"redeyes",_("Do a selection around an eye as precise as possible"),self.cb_tools),
                     ToolPage(_("Blue sky"), "bluesky",_("Click the horizon, a box will appear presenting the sky"), self.cb_tools),
                    )
        # Put all together
        self.notebook.append_page(vbox)
        for vboxtool in vboxtools:
            self.notebook.append_page(vboxtool)
        self.pack1(self.notebook)
        self.pack2(vbox2)

        vbox.show_all()
        vbox2.show_all()
        self.show_all()     

    # Main load function
    def load(self, im):
        gc.collect()
        self.imf = im
        
        # First directly load thumb
        thumb = self.callback("thumb")
        self.draw.load(thumb)
        
        # Load real one
        gobject.idle_add(self.draw.load,im)
        
        self.emit("image-changed", im)

        self.update_label(im)

    def update_label(self, im=None):
        if im == None:
            im = self.draw.imf
        if self.draw.im == None:
            return
        # label
        f = os.path.basename(im)
        w,h = self.draw.im.size
        size = "%sx%s %s" % (w,h,_("pixels"))
        date, tags = self.callback("datetags")
        text = "<b>%s</b> (%s) <i>%s: %s</i>" % (f, size, _("Created on"),str(date))
        if len(tags) > 0:
            text += "\n%s: %s" % (_("Has tags"), str(tags)[1:-1])
        self.desc_label.set_markup(text)
             
    # Callback controls
    def handle_draw_prep(self, widget, event):
        # Change to picture to a small picture in size so it can perform change realtime.
        w = self.draw.get_allocation().width
        h = self.draw.get_allocation().height
        im2 = self.draw.im.copy()
        im2.thumbnail((w,h))
        imo2 = self.draw.imo.copy()
        imo2.thumbnail((w,h))
        self.draw.im = im2
        self.draw.imo = imo2


    def handle_draw_movement(self, widget):
        # Enhancements
        name = widget.get_name()
        
        v = widget.get_value()
        if (v < 105 and v > 100) or (v > 95 and v < 100):
            v = 100
            widget.set_value(100)
        self.draw.enhance(name, v)

        gobject.idle_add(self.draw.do_enhance, False)

    def handle_draw_do(self, widget, event):
        #self.draw.do_enhance()
        self.draw.reload_orig()


    def handle_scale(self, widget, action, scale):
        # Plus/Minus buttons:
        v = scale.get_value()
        v += action*5
        while gtk.events_pending(): 
            gtk.main_iteration()
        scale.set_value(v)
        self.handle_draw_movement(scale)
        #self.handle_draw_do(None, None)

    
    # Handle auto adjustments
    def handle_draw_auto(self, widget):
        name = widget.get_name()
        self.draw.enhance(name, 1)
        self.draw.do_enhance()


    # Callback handle any tools
    def handle_buttons(self, widget):
        name = widget.get_name()
        click = False
        if name == "crop":
            self.notebook.set_current_page(PAGE_CROP)
        if name == "redeyes":
            self.notebook.set_current_page(PAGE_REDEYES)
        if name == "bluesky":
            self.notebook.set_current_page(PAGE_BLUESKY)
            click = True

        self.minibrowser.hide()
        self.draw.tool_init(click=click)

    # Handle undo button
    def handle_draw_reset(self, widget):
        name = widget.get_name()
        # Reset
        if name == "Reset":
            self.draw.reset_enhancements()

    # Exit function
    def sunset(self, widget):
        self.draw.save_to_file()
        self.draw.im = self.draw.imo = self.pixmap = None
        self.callback("sunset_editor")

    # Callback save button
    def save_changes(self, widget, new=False):
        if new:
            dialog = dialogs.MessageInputDialog(markup=_("What should be the new file name?"), entry_text=os.path.basename(self.draw.imf))
            r = dialog.run()
            dialog.destroy()
            if r == gtk.RESPONSE_CANCEL:
                return
            else:
                name = dialog.get_text()
                title, ext = os.path.splitext(self.draw.imf)                   
                if not name.endswith(ext):
                    name += ("."+ext)
                self.draw.save_to_file(path=os.path.dirname(self.draw.imf) + os.sep + name)
        else:
            self.draw.save_to_file()

    # Callback button revert to orgininal
    def revert_to_orig(self, widget):
        self.draw.revert_to_orig()

    # Callback rotate buttons
    def rotate(self, widget, arg):
        if arg == 0:
            self.draw.rotate_left()
        else: self.draw.rotate_right()

    # Callback for zoom buttons
    def zoom(self, widget, arg):
        if arg == 0:
            self.draw.zoom_in()
        else:
            self.draw.zoom_out()

    # Callback redeyes
    def cb_tools(self, args):
        if args == "cancel":
            self.draw.tool_cancel()
            self.notebook.set_current_page(0)
            self.minibrowser.show()
        elif args == "save":
            self.draw.tool_save()
            self.notebook.set_current_page(0)
            self.minibrowser.show()        
        else: 
            self.draw.tool_apply(args)
            
    # Callback scroll
    def scroll_event(self, button, event):
        if self.allow_scroll:
           
            if event.direction == gtk.gdk.SCROLL_UP:
                self.minibrowser.move(None, 1)
            elif event.direction == gtk.gdk.SCROLL_DOWN:    
                self.minibrowser.move(None, -1)
     
    def key_press_event(self, widget, event):
        if event.keyval == 65508 or event.keyval == 65507:
            self.allow_scroll = False
            
    def key_release_event(self, widget, event):
        self.allow_scroll = True   
        

class SunDraw(gtkimageview.ImageView):
    def __init__(self, parent, callback, im=DATADIR+os.sep+"logo.png"):
        gtkimageview.ImageView.__init__(self)

        self.callback = callback
        self.im_modified = False
        self.p = parent
        self.imf = None
        self.im = None

        self.stool = gtkimageview.ImageToolSelector(self)
        self.dtool = gtkimageview.ImageToolDragger(self)

        self.connect("button-release-event", self.tool_click)
        
        self.tool_click_action = False

    def load(self, im):
        self.imf = im

        for scale in self.p.scales:
            scale.set_value(100)

        self.imu = {}
        self.imu_s = []
        self.im_modified = False

        self.reload_orig()    
        self.p.buttonorig.set_sensitive(self.exists_orig()) 

    def reload_orig(self):
        
        # (Re)open the file
        self.im = tools.open_image(self.imf)
        # Convert to RGB, some tools wont work with other modes
        if self.im.mode != "RGB":
            if self.im.mode == "RGBA":
                self.im = tools.convert_rgb(self.im)
            else:
                self.im = self.im.convert("RGB")
        
        self.imo = self.im
        # check for enhancements and refresh
        self.do_enhance()

    def refresh(self):
        
        #pixbuf = gtk.gdk.pixbuf_new_from_file(self.imf)
        self.set_tool(self.dtool)
        
        pixbuf = tools.pil2pixbuf(self.im)  
        self.set_pixbuf(pixbuf)
        #self.imu = {}


    def tool_init(self, click=False):
        self.set_tool(self.stool)
        if click == True:
            self.tool_click_action = True

    def tool_click(self, widget, event):
        if self.tool_click_action == False:
            return False
        x, y = self.get_pointer()
        ph = self.get_draw_rect().height
        my = (self.get_allocation().height-ph)/2
        iw, ih = self.im.size
        y = (y-my)*(float(ih)/ph)
        self.stool.set_selection(gtk.gdk.Rectangle(0,0,iw,y))
        

    def tool_apply(self, tool):
        self.im_modified = True
        rect = self.stool.get_selection()
        x,y,w,h = rect.x, rect.y, rect.width, rect.height
        self.im = tools.TOOLS[tool](self.im, x, y, w, h)
        self.refresh()

    def tool_save(self):
        
        self.imo = self.im
        self.save_to_file() # Save to file to avoid future complications with ajustments, however it doesnot undo anymore
        self.tool_click_action = False

    def tool_cancel(self):
        self.reload_orig()
        self.tool_click_action = False

    def enhance(self, type, v): 
        value = float(v)/100
        if value > 1:
            value = value
        self.imu[type] = value
        if type in self.imu_s:
            self.imu_s.remove(type)
        self.imu_s.append(type)

    def do_enhance(self, progress=True):
        
        im = self.imo

        progress_enabled = progress
        n = len(self.imu)
        
        if n < 1:
            progress_enabled = False
        if progress_enabled:
            progress = utils.SunProgress(_("Processing")+" "+_("enhancements"))
            progress.start(n)
            print "tool - controls:"
        for e in self.imu_s:
            v = self.imu[e]     
            
            im = self.p.controls.do(e, im, v) 
                
            if progress_enabled: 
                print "\t%s = %s" % (e, v*100)
                progress.plus()
                
        if not self.imu == {}:
            self.im_modified = True

        if progress_enabled: progress.destroy()
        self.im = im
        self.refresh()

    
    def reset_enhancements(self, widget=None):
        self.im_modified = False
        for scale in self.p.scales:
            scale.set_value(100)
        self.imu_s = []
        self.imu = {}
        self.im_modified = False
        self.reload_orig()


    def save_to_file(self, path=None):
        if self.im_modified or path != None:

            if path != None:              
                if not os.path.exists(path):
                    file_util.copy_file(self.imf, path)
                else:
                    dialog = gtk.MessageDialog(type=gtk.MESSAGE_WARNING, buttons=gtk.BUTTONS_OK_CANCEL)
                    dialog.set_icon_name("dialog-question")
                    dialog.set_markup(_("This file has been modified\nThis file exists, do you want to overwrite it?"))
                    r = dialog.run()
                    dialog.destroy() 
                    if r == gtk.RESPONSE_CANCEL:
                        return
                    
                self.imf = path

            progress = utils.SunProgress(_("Saving to file"))
            progress.start(2)

            # Create backup
            f = os.path.basename(self.imf)
            rdir = os.path.dirname(self.imf)
            odir = rdir + os.sep + "Originals"
            if not os.path.exists(odir):
                dir_util.mkpath(odir)
            if not os.path.exists(odir + os.sep + f):
                file_util.copy_file(self.imf, odir + os.sep + f)

            progress.plus()
            imexiv = metadata.copy_metadata(self.imf)
            self.im.save(self.imf)
            imexiv.writeMetadata()
     
            progress.plus()
            self.p.buttonorig.set_sensitive(True)
            progress.destroy()
            self.reset_enhancements()

    def revert_to_orig(self, widget=None):
        f = self.imf[self.imf.rfind(os.sep)+1:]
        rdir = self.imf[:self.imf.rfind(os.sep)+1]
        odir = rdir + "Originals"
        if os.path.exists(odir + os.sep + f):
            file_util.copy_file(odir + os.sep + f, self.imf)
            os.remove(odir + os.sep + f)        
        self.p.buttonorig.set_sensitive(False)
        self.reset_enhancements()
        #self.callback("update-thumbs", self.imf)

    def exists_orig(self, *args):
        f = os.path.basename(self.imf)
        rdir = os.path.dirname(self.imf)
        odir = rdir + os.sep + "Originals"
        return os.path.exists(odir + os.sep + f)

    def rotate_right(self, widget=None):
        self.im = tools.rotate_right(self.im)
        self.imo = self.im
        self.im_modified = True
        self.save_to_file()
        self.refresh()

    def rotate_left(self, widget=None):
        self.im = tools.rotate_left(self.im)
        self.imo = self.im
        self.im_modified = True
        self.save_to_file()
        self.refresh()




class ToolPage(gtk.VBox):
    def __init__(self, title, tool, message, cb):
        gtk.VBox.__init__(self)
        self.set_border_width(5)
        self.set_spacing(5)
        self.callback = cb
        
        frame = gtk.Frame(label=title)
        label = gtk.Label()
        label.set_line_wrap(True)
        label.set_size_request(190,100)
        label.set_text(message)
        frame.add(label)
        self.pack_start(frame, expand=False, fill=False)

        button_apply = gtk.Button(stock=gtk.STOCK_APPLY)
        self.apply = button_apply
        button_cancel = gtk.Button(stock=gtk.STOCK_CANCEL)
        button_save = gtk.Button(stock=gtk.STOCK_SAVE)
        button_apply.connect("clicked", self.cb, tool)
        button_cancel.connect("clicked", self.cb, "cancel")
        button_save.connect("clicked", self.cb, "save")
        self.pack_start(button_apply, expand=False, fill=False)
        self.pack_start(button_save, expand=False, fill=False)
        self.pack_start(button_cancel, expand=False, fill=False)   

    def cb(self, widget, name):

        if name == "cancel" or name == "save":
            self.apply.set_sensitive(True)
        else:
            self.apply.set_sensitive(False)
        self.callback(name)

class Plugins:
    def __init__(self):
        self.plugins = {}
        self.controls = tools.Controls()
        
        for folder in PLUGINS_SEARCH_PATH:
            if os.path.exists(folder):
                for f in os.listdir(folder):
                    if f.endswith(".iep"):
                        name = os.path.splitext(f)[0]
                        fo = open(folder + os.sep + f)
                        p = {}
                        # parsing file
                        for l in fo.readlines():
                            if l.startswith("#"):
                                continue                          
                            if "=" in l:
                                vname = l.split("=")[0].strip()
                                value = l.split("=")[1].strip()
                                try: value = int(value)
                                except: pass
                                if value == "None": value= None
                               
                                if vname == "order":
                                    values = []
                                    for v in value.split(","):
                                        values.append(v.strip())
                                    p[vname] = values
                                else:
                                    p[vname] = value
                        # checking whether is correct plugin         
                        if p.has_key("name") and p.has_key("parent") and p.has_key("order") and p.has_key("description"):        
                            self.plugins[name] = p
                         
        
        
                            
    def get_list(self):
        pluginlist = []
        for control in self.plugins:
            plugin = self.plugins[control]
            name = _(plugin["name"].capitalize())
            call = self.do
            args = (control,)
            t = 1
            desc = plugin["description"]
            pluginlist.append((control, name, desc, call, args, t))
        return pluginlist
        
    def do(self, plugin, im, nada):
        
        for control in self.plugins[plugin]["order"]:
            factor = self.plugins[plugin][control]
            factor = float(factor)/100
            if factor > 1:
                factor = factor
            im = self.controls.do(control, im, factor)
        return im    
