#!/usr/bin/env python
#
#   Copyright (C) 2008 Alberto Piai <alberto.piai@gmail.com>
#
#   This program is free software: you can redistribute it and/or modify
#   it under the terms of the GNU Affero General Public License as
#   published by the Free Software Foundation, either version 3 of the
#   License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU Affero General Public License for more details.
#
#   You should have received a copy of the GNU Affero General Public License
#   along with this program.  If not, see <http://www.gnu.org/licenses/>.

import gtk
import gobject

class Cache(object):
    """
    This class implements an image cache.
    """

    _max_cache_size = 5
    
    def __init__(self, loader=None):
        """
        loader: a callable which takes a filename and returns a gtk.gdk.Pixbuf
        """
        self._cache = {}
        self._cached_keys = []
        if loader:
            self._load = loader
        else:
            self._load = self._standard_loader
    
    def _standard_loader(self, filename):
        """
        A simple loader.
        """
        return gtk.gdk.pixbuf_new_from_file(filename)
    
    def get(self, filename):
        """
        Get a picture from the cache or load it from disk.
        """
        try:
            pixbuf = self._cache[filename]
            #print "imagecache: cache hit ", filename
            return pixbuf
        except (KeyError):
            #print "imagecache: cache miss"
            return self._load(filename)
    
    def preload(self, filename):
        """
        Preload an image in the cache
        """
        def _preload():
            if len(self._cache) > self._max_cache_size:
                oldest = self._cached_keys[0]
                #print "imagecache: removing oldest item from cache, ", oldest
                self._cache.pop(oldest)
                self._cached_keys.remove(oldest)
            if not self._cache.has_key(filename):
                #print "imagecache: loading %s into the cache" % (filename,)
                self._cache[filename] = self._load(filename)
                self._cached_keys.append(filename)
        gobject.idle_add(_preload)

