#!/usr/bin/env python
#
#   This file is part of IntiPunku
#   Copyright (C) 2007 Mark Baas <mark.baas123@gmail.com>
#
#   This program is free software: you can redistribute it and/or modify
#   it under the terms of the GNU Affero General Public License as
#   published by the Free Software Foundation, either version 3 of the
#   License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU Affero General Public License for more details.
#
#   You should have received a copy of the GNU Affero General Public License
#   along with this program.  If not, see <http://www.gnu.org/licenses/>.

from inti import exiv2
import os, time, datetime
from inti.constants import *

def get_tags(taim):
    tags = []
    if type(taim['Iptc.Application2.Keywords']) == type(""): 
        if taim['Iptc.Application2.Keywords'].strip() != "":
            tags = [taim['Iptc.Application2.Keywords']]
    else: 
        for key in taim['Iptc.Application2.Keywords']:
            if key.strip() != "":
                tags.append(key)
    return tags

def get_datetags(path):
    image = exiv2.Image(path)
    image.readMetadata()
    if "Exif.Image.DateTime" in image.keys():
        datetimetag = "Exif.Image.DateTime"
    else: datetimetag = "Exif.Photo.DateTimeOriginal"
    
    if datetimetag in image.keys():
        idatetime = image[datetimetag]
        if type(idatetime) == type(str()):
            statinfo = os.stat(path)
            year,month,day,x1,x2,x3,x4,x5,x6 = time.gmtime(statinfo.st_ctime)
            date = datetime.datetime.fromtimestamp(statinfo.st_ctime)
        else:
            date = idatetime
            
    else:
        statinfo = os.stat(path)
        year,month,day,x1,x2,x3,x4,x5,x6 = time.gmtime(statinfo.st_ctime)
        date = datetime.datetime.fromtimestamp(statinfo.st_ctime)

    if 'Iptc.Application2.Keywords' in image.keys():
        tags = get_tags(image)
    else: tags = []
    return date, tags

def set_date(path, dt):
    image = exiv2.Image(path)
    image.readMetadata()
    image["Exif.Image.DateTime"] = dt
    image["Exif.Image.Software"] = "IntiPunku"
    image.writeMetadata()

def has_exifdatetime(path):
    image = exiv2.Image(path)
    image.readMetadata()
    keys = image.keys()
    if "Exif.Image.DateTime" in keys or "Exif.Photo.DateTimeOriginal" in keys:
        return True
    return False
    
    
def copy_metadata(path1, path2=None):
    # path2 is none, cache the keys
    image = exiv2.Image(path1)
    image.readMetadata()
    if path2 == None:
        image["Exif.Image.Software"] = "IntiPunku"
        return image
    else:
        image2 = exiv2.Image(path2)
        image2.readMetadata()
        image.copyMetadataTo(image2)
        #image2.writeMetadata()
        return None


def write_tags(path, tags):
    image = exiv2.Image(path)
    image.readMetadata()
    image["Iptc.Application2.Keywords"] = tags
    image["Iptc.Application2.Program"] = "Intipunku"
    image["Iptc.Application2.ProgramVersion"] = VERSION
    image.writeMetadata()

def reload_tags(tagindex):
    n = len(tagindex)
    for im in tagindex:
        while gtk.events_pending(): 
            gtk.main_iteration() 
        image = exiv2.Image(im)
        image.readMetadata()
        if 'Iptc.Application2.Keywords' in image.iptcKeys():
            get_tags(image)
        else: tags = []
        tagindex[im] = tags

    return tagindex
