#!/usr/bin/env python
#
#   This file is part of IntiPunku
#   Copyright (C) 2007 Mark Baas <mark.baas123@gmail.com>
#
#   This program is free software: you can redistribute it and/or modify
#   it under the terms of the GNU Affero General Public License as
#   published by the Free Software Foundation, either version 3 of the
#   License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU Affero General Public License for more details.
#
#   You should have received a copy of the GNU Affero General Public License
#   along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
#
# Optional components are pano, gdata, gphoto2
# There should be made wrappers for this

from inti import utils
import gtk
import os, os.path
import tempfile

# Localization
import gettext, locale
_ = gettext.gettext

optionals = {
    "gdata" : [
            { 
                "apt" : "python-gdata python-gconf python-gnome2-desktop",
            },
            ],
    "gphoto" : [
            {
                "apt" : "gphoto2",
            },
            ],
    "pano" : [
            {
                "apt" : "hugin-tools enblend panomatic",
            },
            ],
    "flickr" : [
            {
                "apt" : "python-flickr",
            },
            ],
    "ming" : [
            {
                "apt" : "python-ming",
            },
            ],
            
}

if os.path.exists("/etc/debian_version"): 
    cmd = "gksudo 'apt-get -y --force-yes install PACKAGE'"
    system = "apt"
elif os.path.exists("/etc/redhat-release"): 
    cmd = None
    system = "yum"
else: cmd = None

# Unsupported package manager
def unsupported(module):
    dialog = gtk.MessageDialog(flags=gtk.DIALOG_MODAL, type=gtk.MESSAGE_ERROR, buttons=gtk.BUTTONS_OK)
    dialog.set_markup(_("Sorry your package manager is unsupported.\nPlease install the follow package manually '%s'" % (module)))
    dialog.set_title(_("Package manager unsupported"))
    r = dialog.run()
    dialog.destroy()  
    
def check_repo():
    if system == "apt":
        f = open("/etc/apt/sources.list")
        lines = f.readlines()
        f.close()
        repo = "deb http://ppa.launchpad.net/intipunku/ubuntu hardy main"
        for l in lines:
            if l.startswith(repo):
                return True
        if os.path.exists("/etc/apt/source.list.d/intipunku.list"): return True
        # Repo no added
        dialog = gtk.MessageDialog(flags=gtk.DIALOG_MODAL, type=gtk.MESSAGE_ERROR, buttons=gtk.BUTTONS_OK)
        dialog.set_markup(_("Sorry the intipunku apt repository is not added to you /etc/apt/sources.list\n\nPlease add the following lines to /etc/apt/sources.list:\n\n<b>%s</b>" % (repo)))
        dialog.set_title(_("APT repo not installed"))
        r = dialog.run()
        dialog.destroy()  
            

# Install module and reload
def install(module):
    if cmd == None:
        unsupported(module)
        return False
    if not check_repo(): return False
    package = optionals[module][0][system]
    cmd2 = cmd.replace("PACKAGE",package)
    os.system(cmd2)        
    return True

# Check whether to install module
def question(c):
    dialog = gtk.MessageDialog(flags=gtk.DIALOG_MODAL, type=gtk.MESSAGE_QUESTION, buttons=gtk.BUTTONS_OK_CANCEL)
    dialog.set_markup(_("This component depends on external resources, which are not installed.\nWould you like to install dependencies to enable this feature?"))
    dialog.set_title(_("Component disabled!"))
    r = dialog.run()
    dialog.destroy()
    if r == -5:
        return True
    return False
    


def check(c, callback, constant):
    while not constant:
        r = question(c)       
        if r:
            r2 = install(c)
            if r2 == False:
                return False
            constant = callback()
        else:
            return False
    return True 
    
    



            



