#!/usr/bin/env python
#
#   This file is part of IntiPunku
#   Copyright (C) 2007 Mark Baas <mark.baas123@gmail.com>
#
#   This program is free software: you can redistribute it and/or modify
#   it under the terms of the GNU Affero General Public License as
#   published by the Free Software Foundation, either version 3 of the
#   License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU Affero General Public License for more details.
#
#   You should have received a copy of the GNU Affero General Public License
#   along with this program.  If not, see <http://www.gnu.org/licenses/>.

import gtk, gobject, cairo, math
from PIL import Image
import gc
from inti import utils
import imagecache

# Localization
import gettext
_ = gettext.gettext

class Slideshow(gtk.Window):
    def __init__(self, files, config):
        gtk.Window.__init__(self)
        self.set_app_paintable(True)
        self.fullscreen()
        self.w, self.h = gtk.gdk.screen_width(),gtk.gdk.screen_height()
        self.realize()        
        self.window.set_background(gtk.gdk.Color(red=0, green=0, blue=0, pixel=0))
        self.gim = gtk.Image()
        self.add_events(gtk.gdk.POINTER_MOTION_MASK | gtk.gdk.SCROLL_MASK)

        self.connect('scroll-event', self.scroll_event)
        self.connect("motion-notify-event", self.reset_timer)
        
        
        b_bac = gtk.Button()
        b_bac.set_image(gtk.image_new_from_pixbuf(utils.load_icon("back")))
        b_bac.set_relief(gtk.RELIEF_NONE)
        b_bac.connect("clicked", self.go_backward)
        b_bac.set_sensitive(False)
        self.back = b_bac
        #b_bac.set_property("label", None)
        
        b_exit = gtk.Button(stock=gtk.STOCK_QUIT)
        b_exit.set_relief(gtk.RELIEF_NONE)
        b_exit.connect("clicked", self.sunset)

        b_for = gtk.Button()
        b_for.set_image(gtk.image_new_from_pixbuf(utils.load_icon("forward")))
        b_for.set_relief(gtk.RELIEF_NONE)
        b_for.connect("clicked", self.go_forward)
        self.forward = b_for
        #print b_for.get_children()[0].get_children()[0].get_children()
        

        # first column
        vbox1=gtk.VButtonBox()
        c = [None,b_bac, b_exit]
        for i in c:
            if i == None:
                i = gtk.Label()
            vbox1.pack_start(i)
        # second
        vbox2=gtk.VButtonBox()
        c = [None, b_for, None]
        for i in c:
            if i == None:
                i = gtk.Label()
            vbox2.pack_start(i)

        hbox = gtk.HBox()
        hbox.pack_start(vbox1, fill=False, expand=False)
        hbox.pack_start(self.gim)
        hbox.pack_start(vbox2, fill=False, expand=False)
        self.add(hbox)

        self.photos = files
        self.cfile = 0
        self.show()
        self.w_max = self.w - 200
        self.set_image()
        
        self.count = 0
        self.config = config
        if self.config["slide_auto"]:
            gobject.timeout_add(1000, self.timer_up)
        
        self.cache = imagecache.Cache(loader=self._get_pixbuf_from_file)
        
    def set_image(self): 
        gobject.idle_add(self._set_image)
    
    def _set_image(self):
        f = self.photos[self.cfile]
        print "sliding", f
        pixbuf = self.cache.get(f)
        self.gim.set_from_pixbuf(pixbuf)
        try:
            self.cache.preload(self.photos[self.cfile+1])
        except IndexError:
            pass
    
    def _get_pixbuf_from_file(self, f):
        pixbuf = gtk.gdk.pixbuf_new_from_file(f)
        r = float(self.h) / pixbuf.get_height()
        w = int(r*pixbuf.get_width())
        print w, self.w_max
        if w > self.w_max:
            w=self.w_max
            r = float(self.w_max)/pixbuf.get_width()
            h = int(r*pixbuf.get_height())
        else:
            h = self.h
        print w, h
        return pixbuf.scale_simple(w, h, gtk.gdk.INTERP_BILINEAR)

    def go_forward(self, widget):
        if self.forward.get_property("sensitive"):
            self.cfile += 1
            self.do_buttons()
            self.set_image()
                
    def go_backward(self, widget):
        if self.back.get_property("sensitive"):
            self.cfile -= 1
            self.do_buttons()
            self.set_image()

    def do_buttons(self):
        self.back.set_sensitive(not self.cfile == 0)
        self.forward.set_sensitive(not self.cfile == len(self.photos)-1)
        
    def scroll_event(self, button, event):
        if event.direction == gtk.gdk.SCROLL_UP:
            self.go_forward(None)
        elif event.direction == gtk.gdk.SCROLL_DOWN:
            self.go_backward(None)
            
    def timer_up(self, *args):
        if self.count == None:
            return False
        self.count += 1
        if self.count == int(self.config["slide_timeout"]):
            self.go_forward(None)        
            self.count = 0
        return True
        
    def reset_timer(self, *args):
        self.count = 0

    def sunset(self, *args):
        self.count = None
        self.destroy()


