#!/usr/bin/env python
#
#   This file is part of IntiPunku
#   Copyright (C) 2007 Mark Baas <mark.baas123@gmail.com>
#
#   This program is free software: you can redistribute it and/or modify
#   it under the terms of the GNU Affero General Public License as
#   published by the Free Software Foundation, either version 3 of the
#   License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU Affero General Public License for more details.
#
#   You should have received a copy of the GNU Affero General Public License
#   along with this program.  If not, see <http://www.gnu.org/licenses/>.

from PIL import Image, ImageEnhance, ImageFilter, ImageDraw, ImageOps, ImageFile
import os, datetime, sys, os.path, time
from distutils import dir_util, file_util
from inti import utils
from inti import ellipse
import math
import gtk
from threading import Thread
import StringIO
from inti.constants import *
import colorsys
import tempfile

# Localization
import gettext
_ = gettext.gettext

# Check for pano (optional)
def load_pano():
    atools = ["panomatic", "autooptimiser", "nona", "enblend"]
    for tool in atools:
        if not utils.inpath(tool): return False
    return True
PANO_ENABLED = load_pano()


if sys.platform.startswith("win"):
    import ctypes
    from win32con import *

def open_image(path):
    im = Image.open(path)

    return im

def crop(im, x1, y1, w, h):
        i = im.crop((x1,y1,x1+w,y1+h))
        return i

##################
# Controls & auto tools

def hue(im, factor):
    # ruins luminance
    factor = (factor-1)*255
    im1 = im.convert("L").convert("RGB")
    mag = math.sqrt(2.0)
    xrs = 1.0/mag
    xrc = 1.0/mag
    im1 = im1.convert("RGB",
        (1, 0, 0, 0,
        0, xrc, xrs, 0,
        0, -xrs, xrc, 0))
    
    mag = math.sqrt(3.0)
    yrs = -1.0/mag
    yrc = math.sqrt(2.0)/mag
    im1 = im1.convert("RGB",
        (yrc, 0, -yrs, 0,
        0, 1, 0, 0,
        yrs, 0, yrc, 0))   
        
    zrs = math.sin(factor*math.pi/180.0)
    zrc = math.cos(factor*math.pi/180.0)
    
    im1 = im1.convert("RGB",
        (zrc, zrs, 0, 0,
        -zrs, zrc, 0, 0,
        0, 0, 1, 0))
        
    im1 = im1.convert("RGB",
        (yrc, 0, yrs, 0,
        0, 1, 0, 0,
        -yrs, 0, yrc, 0) )
        
    im1 = im1.convert("RGB",
        (1, 0, 0, 0,
        0, xrc, -xrs, 0,
        0, xrs, xrc, 0))
        
    return im1
    
def saturation(im, factor):
    imo = im.copy()
    
    factor /= 2
    matrix = (
        (1.0-factor), (1.0-factor), (1.0-factor), 0,
        (1.0-factor), (1.0-factor), (1.0-factor), 0, 
        (1.0-factor), (1.0-factor), (1.0-factor), 0, 
        )
    im1 = im.convert("RGB", matrix)
    return im1
        
    

def color(im,factor):   
    enhancer = ImageEnhance.Color(im)
    i = enhancer.enhance(factor)
    return i

def brightness(im, factor):   
    enhancer = ImageEnhance.Brightness(im)
    i = enhancer.enhance(factor)
    return i

def contrast(im, factor):
    enhancer = ImageEnhance.Contrast(im)
    i = enhancer.enhance(factor)
    return i    

def temperature(im, factor):
    #50x 
    r, g, b = im.split()

    mask = r.point(lambda i: i <= 255 and 255)
    out = r.point(lambda i: i * factor)
    r.paste(out, None, mask)
    mask = g.point(lambda i: i <= 255 and 255)
    out = g.point(lambda i: i * (factor))
    g.paste(out, None, mask)
    im = Image.merge(im.mode, (r,g,b))
    
    return im

def sepia(im, factor):
    matrix = (
    0.393, 0.769, 0.189,0,
    0.349, 0.686, 0.168,0,
    0.272, 0.534, 0.131,0)
    return im.convert("RGB", matrix)

def sepia2(im, factor):
    im = color(im , 0)
    imsepia = Image.new("RGB", im.size, (112,66,20))
    im = Image.blend(im, imsepia, 0.5) 
    
    im = contrast(im, 1.14)
    im = brightness(im, 1.14)
    return im

def autocontrast(im, factor):
    return ImageOps.autocontrast(im, 1)

def equalize(im, factor):
    return ImageOps.equalize(im)


def redeye(im, x, y, w, h):
    wi, he = im.size
    x = x + (w/2)
    y = y + (h/2)
    return ellipse.ellipse(im, (x,y), w, h, lambda color: (color[1],color[1],color[2]))

def bluesky(im, x,y,w,h):
    width,height = im.size
    o = 0.5    
    g = 0.40
    # Create 2 a one color blue image
    sky = im.crop((0,0,width,h))
    sky = sky.convert("RGBA")
    sky1 = Image.new("RGBA",(width,round(h*(1-g))),(143,181,223))
    sky2 = Image.new("RGBA",(width,round(h*g)),(143,181,223))

    # Gradient
    # create a vertical gradient...
    gradient = Image.new('L', (1,o*255))
    for p in range(o*255):
        gradient.putpixel((0,(o*255-1)-p),p)
    alpha = gradient.resize((width,round(h*g)))
    # put alpha in the alpha band of im...
    #sky2.putalpha(alpha)
    
    sky.paste(sky1, (0,0,width,round((1-g)*h)), Image.new("L", sky1.size, (o*255)))
    sky.paste(sky2, (0,round((1-g)*h),width,h), alpha)
    im.paste(sky, (0,0,width,h))
    return im
    
def grayscale(im, factor=None):
    return color(im, 0)
    
def negative(im, factor=None):
    return ImageOps.invert(im)
    
def dodge(a, b, alpha):
    return min(int(a*255/(256-b*alpha)), 255)    

def drawing(im, factor=None):
    # Copyright (c) 2008 the authors listed at the following URL, and/or
    # the authors of referenced articles or incorporated external code:
    # http://en.literateprograms.org/Photo_to_drawing_(Python)?action=history&offset=20070404131120

    blur = 10
    alpha = 1.0
    # setup layers
    im1 = im.convert("L")
    size = im1.size
    if min(size[0], size[1]) > 1000:
        # it needs to be resize, otherwise the pixel iteration takes too long, the benefit is that the effect actually is stronger.
        im1.thumbnail((int(float(size[0])/2),int(float(size[1])/2)))
    im2 = im1.copy()
    im2 = ImageOps.invert(im2)
    for i in range(blur):
        im2 = im2.filter(ImageFilter.BLUR)
        
    width, height = im2.size
    pixels1 = im1.load()
    pixels2 = im2.load()
    # Pixel iteration
    for x in range(width):
        for y in range(height):
            a = pixels1[x,y]
            b = pixels2[x,y]       
            # Dodge it         
            pixels1[x,y] = dodge(a, b, alpha)

    # Return to RGB and it's original size
    im = im1.convert("RGB").resize(size)        
    return im
    
def charcoal(im, factor=None):
    
    im = equalize(im, 1)
    im = negative(im, 1)
    
    # Setup layers
    im1 = im.convert("L")
    im2 = Image.new("RGB", im.size, (220, 206, 85))

    im1 = ImageOps.invert(im1.filter(ImageFilter.FIND_EDGES))
    
    im1 = contrast(im1, 2)
    
    
    for i in range(2):
        im1 = im1.filter(ImageFilter.BLUR)
    im = Image.blend(im2, im1.convert("RGB"), 0.7)
    
    im1 = contrast(im1, 1.9)
    
    return im
    
    
    
def hue4(im, factor):
    factor = ((float(factor)/2))
    s = 1.0
    v = 1.0
    h = factor
    print h,s,v,"=>"
    r,g,b = colorsys.hsv_to_rgb(h, s, v)
    print r,g,b
    r *= 255
    g *= 255
    b *= 255
    print r*255,g*255,b*255
    
    return Image.new("RGB", im.size, (r,g,b))
    
def red(im, factor):
    #50x 
    r, g, b = im.split()

    mask = r.point(lambda i: i <= 255 and 255)
    out = r.point(lambda i: i * factor)
    r.paste(out, None, mask)
    im = Image.merge(im.mode, (r,g,b))
    
    return im
    
def green(im, factor):
    #50x 
    r, g, b = im.split()

    mask = g.point(lambda i: i <= 255 and 255)
    out = g.point(lambda i: i * factor)
    g.paste(out, None, mask)
    im = Image.merge(im.mode, (r,g,b))
    
    return im
    
def blue(im, factor):
    #50x 
    r, g, b = im.split()

    mask = b.point(lambda i: i <= 255 and 255)
    out = b.point(lambda i: i * factor)
    b.paste(out, None, mask)
    im = Image.merge(im.mode, (r,g,b))
    
    return im
    
def blur(im, factor):
    for i in range(2):
        im = im.filter(ImageFilter.BLUR)
    return im

    
# End controls & autotools
###########################33

def rotate_real(path, deg):
    im = Image.open(path)
    im = im.rotate(deg, Image.ANTIALIAS)
    return im

def rotate(path,rl):
    im = Image.open(path)
    if rl == "right":
        im = rotate_right(im)
    if rl == "left":
        im = rotate_left(im)

    f = os.path.basename(path)
    rdir = os.path.dirname(path)
    odir = rdir + os.sep + "Originals"
    if not os.path.exists(odir):
        dir_util.mkpath(odir)
    if not os.path.exists(odir + os.sep + f):
        file_util.copy_file(path, odir + os.sep + f)
    im.save(path)

def rotate_right(im):
    return im.transpose(Image.ROTATE_270)


def rotate_left(im):
    return im.transpose(Image.ROTATE_90)


def shadowx(image, offset=(5,5), background=0xffffff, shadow=0x444444, 
                border=5, iterations=3):
    """
    Add a gaussian blur drop shadow to an image.  

    image       - The image to overlay on top of the shadow.
    offset      - Offset of the shadow from the image as an (x,y) tuple.  Can be
                positive or negative.
    background  - Background colour behind the image.
    shadow      - Shadow colour (darkness).
    border      - Width of the border around the image.  This must be wide
                enough to account for the blurring of the shadow.
    iterations  - Number of times to apply the filter.  More iterations 
                produce a more blurred shadow, but increase processing time.
    """
    
    # Create the backdrop image -- a box in the background colour with a 
    # shadow on it.
    totalWidth = image.size[0] + abs(offset[0]) + 2*border
    totalHeight = image.size[1] + abs(offset[1]) + 2*border
    back = Image.new("RGBA", (totalWidth, totalHeight), (0,0,0,0))
    if image.mode != "RGBA": 
        image = image.convert("RGBA")
    

    # Place the shadow, taking into account the offset from the image
    shadowLeft = border + max(offset[0], 0)
    shadowTop = border + max(offset[1], 0)
    back.paste(shadow, [shadowLeft, shadowTop, shadowLeft + image.size[0], 
    shadowTop + image.size[1]] )

    # Apply the filter to blur the edges of the shadow.  Since a small kernel
    # is used, the filter must be applied repeatedly to get a decent blur.
    n = 0
    while n < iterations:
        back = back.filter(ImageFilter.BLUR)
        n += 1

    # Paste the input image onto the shadow backdrop  
    imageLeft = border - min(offset[0], 0)
    imageTop = border - min(offset[1], 0)
    back.paste(image, (imageLeft, imageTop))
    
    return back

def shadow(im):
    if im.mode != "RGBA": im = im.convert("RGBA")
    width = im.size[0]
    height = im.size[1]
    back = Image.new("RGBA", (width+18, height+18), (0,0,0,0))
    shadow = Image.new("RGBA", (width,height), (0,0,0,150))

    back.paste(shadow, (10,10)) 
    iterations = 3
    for i in range(iterations):
        back = back.filter(ImageFilter.BLUR)

    back.paste(im, (5,5))
    back.save("/tmp/thumb.png", "PNG")
    return back

def fit(im):
    w, h = im.size
    #w = w*0.9
    #h = h*0.9

    px = 0.1*w/2
    py = 0.1*h/2
    return ImageOps.fit(im, (w,h), Image.NEAREST, 0.01, (0.1,0.1))

def pano(images):     

    progress = utils.SunProgress(_("Creating panorama, this may take a while..."), cancel=True, icon="panorama") # Progress show
    key_list = []
    tmp = tempfile.mkdtemp(prefix="intipano")
    project = tmp + os.sep + "panorama.pto"

    tiffs = []
       
    progress.start(5)
    progress.plus()
    while 1:
        # Step one calculate keypoint and create pto file    
        if progress.cancel: break # Check if cancel
        progress.set_text(_("Generating keypoints")) # text progress
        image_list_str = ""
        for imf in images:
            image_list_str += "\"%s\" " % imf
        #arg="autopano-sift-c \"%s\" %s" % (project, image_list_str.strip())
        arg="panomatic -o \"%s\" %s" % (project, image_list_str.strip())
        progress.pulse()
        utils.sunthread(os.popen, (arg,))
       

        # Step two autooptimize
        if progress.cancel: break # Check if cancel
        progress.set_text(_("Optimizing configuration")) # text progress
        os.chdir(tmp)
        arg = "autooptimiser -o %s -als %s" % (project, project)
        progress.pulse()
        utils.sunthread(os.popen, (arg,))
        progress.plus() # Add progress
        
        # Change output type to tiff
        f = open(project)
        data = f.read().replace("n\"JPEG q90\"", "n\"TIFF_m\"")
        f.close()
        f =open(project, "w")
        f.write(data)
        f.close()
        if not "TIFF_m" in data: 
            utils.report_error(_("Some error has occured, please try again")); break
            
        progress.plus() # Add progress

        # Step three nona
        if progress.cancel: break # Check if cancel
        progress.set_text(_("Processing panorama")) # text progress

        name1 = os.path.splitext(os.path.basename(images[0]))[0]
        name2 = os.path.splitext(os.path.basename(images[-1]))[0]
        fname = "pano_" + name1 + "-" + name2
        # dangerous because, if of something it get stuck in loop
        tries = 0     
        while os.path.exists(os.path.join(os.path.dirname(images[0]), fname + ".jpg")) and not tries > 10:
            print os.path.exists(fname) or not tries > 10
            fname += "_"
            tries += 1
        
        if tries > 10:
            utils.report_error(_("Some error has occured, please try again")); break
            
        arg = "nona -o \"%s\" \"%s\"" % (os.path.join(tmp, fname + ".tif"), project)

        progress.pulse()
        utils.sunthread(os.popen, (arg,))
        progress.plus() # Add progress
        
        # Step four enblend
        if progress.cancel: break # Check if cancel
           
        progress.set_text(_("Blending panorama")) # text progress
        arg = "enblend -o \"%s\"" % (os.path.join(tmp, fname + ".tif"))
        for tiff in os.listdir(tmp):
            if tiff.endswith(".tif"):
                arg += " \"%s\"" % (os.path.join(tmp, tiff)) 
        progress.pulse()     
        utils.sunthread(os.popen, (arg,))
        im = open_image(os.path.join(tmp, fname + ".tif"))
        output = os.path.join(os.path.dirname(images[0]),fname + ".jpg")
        im.save(output, "JPEG")
        del im
        
        break

    # Clean up
    print "clean up tempfiles panorama"
    # tmpfiles are project.pto, tiff files
    dir_util.remove_tree(tmp)
    print "panorama creation finished/failed"
    progress.destroy()
    return output

def create_thumb(path):
    im = Image.open(path)
    im.thumbnail((148,148), Image.BILINEAR)
    if im.mode != "RGBA":
        im = im.convert("RGBA")
    im = shadow(im)
   
    return im

def refresh_thumb(path, thpath):
    im = Image.open(path)
    im.thumbnail((148,148))
    im = shadow(im)
    f = thpath
    im.save(f, "PNG")

def resize_image(path, topath, size, type="JPEG"):
    im = Image.open(path)
    im.thumbnail(size, Image.ANTIALIAS)
    im.save(topath, type)

def set_wallpaper(path):
    if sys.platform.startswith("win"):
        im = Image.open(path)
        im.thumbnail((gtk.gdk.screen_width(),gtk.gdk.screen_height()))
        im.save("\wallpaper.bmp","BMP")
        # complicated
        #SPI_SETDESKWALLPAPER = 20 # According to http://support.microsoft.com/default.aspx?scid=97142
        ctypes.windll.user32.SystemParametersInfoA(SPI_SETDESKWALLPAPER, SPIF_UPDATEINIFILE, "\wallpaper.bmp" , SPIF_SENDWININICHANGE)
        os.remove("\wallpaper.bmp")
    elif MAC:
        os.system("set_wallpaper.sh \"%s\"" % path)
    else:
        # Gnome/KDE?
        f = os.popen("ps aux", "r")
        for l in f.readlines():
            if " nautilus " in l:
                # i think gnome is running
                os.system("gconftool --type=string --set /desktop/gnome/background/picture_filename " + '"' + path + '"')
            if " kdeinit " in l:
                # i think kde is running
                os.system("dcop kdesktop KBackgroundIface setWallpaper " + '"' + path + '" ' + "6")

# Constant for all tools that use selections
TOOLS = { "crop":crop,
          "redeyes":redeye,
          "bluesky":bluesky,
        }



def convert_image(path, path2):
    im = open_image(path)
    im.save(path2, "JPEG")

def convert_rgb(im):
    back = Image.new("RGBA", (im.size[0],im.size[1]), (255,255,255,0))   
    r,g,b,a = im.split()
    back.paste(im, (0,0), a)
  
    return back.convert("RGB")

def convert_raw(path, path2):
    fd = os.popen("dcraw -T -c %s" % path1)
    f = StringIO.StringIO(fd.read())
    im = Image.open_image(f)
    im.save(path2, "JPEG")
    
    
def pil2pixbuf(im):
    if im.mode == "RGBA":
        pildec = "png"
        pbdec = "png"
    else:
        pildec = "ppm"
        pbdec = "pnm"        
    file = StringIO.StringIO ()
    im.save(file, pildec)
    contents = file.getvalue()
    file.close ()
    loader = gtk.gdk.PixbufLoader(pbdec)
    loader.write(contents, len (contents))
    pixbuf = loader.get_pixbuf()
    loader.close()
    return pixbuf
    
def pixbuf_draw_rect(f, color="#000000"):
    if len(color) > 7:
        colorn = "#"
        c = 0
        for a in color[1:]:
            c += 1
            if c <= 2:
                colorn += a
            if c == ((len(color)-1)/6+2): 
                c = 0
        color = colorn
    im = Image.open(f)
    draw = ImageDraw.Draw(im)
    draw.rectangle((1,1,im.size[0]-1,im.size[1]-1), outline=color)
    draw.rectangle((2,2,im.size[0]-2,im.size[1]-2), outline=color)
    return pil2pixbuf(im)

class Controls:
    def __init__(self):
        """
        In this class all the control go
        """
        
        self.caps = {"color": (_("Color"), _("Increases/decreases colors"), color, None, 0),
                    "brightness": (_("Brightness"), _("Increases/decreases brightness"), brightness, None, 0),
                    "contrast": (_("Contrast"), _("Increases/decreases contrast"), contrast, None, 0),
                    "temperature": (_("Temperature"), _("Will make the picture look colder/warmer"), temperature, None, 0),
                    "sepia": (_("Sepia"), _("Sepia tone refers to the coloring of a black and white photographic print"), sepia, None, 1),
                    "autocontrast": (_("Auto-contrast"), _("Maximize (normalize) image contrast"), autocontrast, None, 1),
                    "equalize": (_("Equalize"), _("Equalize the image histogram"), equalize, None, 1),
                    "grayscale": (_("Grayscale"),_("Converts image into grayscale"), grayscale, None, 1),
                    "negative": (_("Negative"), _("Negates image"), negative, None, 1),
                    "pencil2": (_("Pencil"), _("Transforms a photograph to resemble a grayscale pencil drawing"), drawing, None, 1),
                    "charcoal": (_("Charcoal"), _("Transforms a photograph to resemble a charcoal drawing"), charcoal, None, 1),
                    #"hue":  (_("Hue"), _("Increases/decreases hue"), hue2, None, 0),
                    "red": (_("Red"), _("Increases/decreases red"), red, None, 0),
                    "green": (_("Green"), _("Increases/decreases green"), green, None, 0),
                    "blue": (_("Blue"), _("Increases/decreases blue"), blue, None, 0),
                    "blur": (_("Blur"), _("Blurs a picture"), blur, None, 1),
                    #"saturation":  (_("Saturation"), _("Increases/decreases saturation"), saturation, None, 0),
                    }
                    
        self.caps_s = ["brightness", "contrast", "temperature", "color", "red", "green", "blue", "sepia", "autocontrast", "equalize", "grayscale", "blur", "negative", "pencil2", "charcoal"]
                    
    def add_capability(self, control, name, desc, callback, args, t):
        print "tools: adding id %s name %s type %s" % (control, name, t) 
        self.caps[control] = (name, desc, callback, args, t)
        self.caps_s.append(control)
    
    def get_controls(self):
        controls = []
        for cap in self.caps_s:
            t = self.caps[cap][4]
            if t == 0:
                name = self.caps[cap][0]
                desc = self.caps[cap][1]
                controls.append((cap, name, desc))
        return controls
                
    def get_autotools(self):
        controls = []
        for cap in self.caps_s:
            t = self.caps[cap][4]
            if t == 1:
                name = self.caps[cap][0]
                desc = self.caps[cap][1]
                controls.append((cap, name, desc)) 
        return controls                   
                
    def do(self, control, im, factor):
        args = self.caps[control][3]
        
        if args == None:
            return self.caps[control][2](im, factor)
        else:
            args = args + (im, factor)
            return self.caps[control][2](*args)
