#!/usr/bin/env python
#
#   This file is part of IntiPunku
#   Copyright (C) 2007 Mark Baas <mark.baas123@gmail.com>
#
#   This program is free software: you can redistribute it and/or modify
#   it under the terms of the GNU Affero General Public License as
#   published by the Free Software Foundation, either version 3 of the
#   License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU Affero General Public License for more details.
#
#   You should have received a copy of the GNU Affero General Public License
#   along with this program.  If not, see <http://www.gnu.org/licenses/>.

import sys
from distutils import dir_util, file_util
import os
from inti import utils
from inti import tools
from inti import dialogs

def load_gdata():
    try:
        import gdata.photos
        import gdata.photos.service
        import gnomekeyring
        global gdata, gnomekeyring
        return True
    except StandardError, err:
        print err
        return False
        
def load_flickr():
    try:
        import flickrapi
        global flickrapi
        return True
    except:
        return False
        
GDATA_ENABLED = load_gdata()
FLICKR_ENABLED = load_flickr()

import gobject, gtk



#import facebook, mozembed

import tempfile
from PIL import Image

from inti.constants import *


# Localization
import gettext
_ = gettext.gettext


def get_picasa_login(config):
    keyring = gnomekeyring.get_default_keyring_sync()
    auth_token = int(config["picasa_token"])
    if auth_token > 0:
        try:
            secret = gnomekeyring.item_get_info_sync(keyring, auth_token).get_secret()
        except gnomekeyring.DeniedError, err:
            print err
            login = None
            password = None
            auth_token = 0
        else:
            login, password = secret.split('\n')
            return login, password
    else:
        login = None
        password = None

    if login != None and password != None:
        return login, password
    pws = gdata.photos.service.PhotosService()
    message = ""
    while 1:
        dialog = dialogs.LoginDialog(login, password, message)
        response = dialog.run()
        login = dialog.get_login()
        password = dialog.get_password()
        dialog.destroy()
        if response == gtk.RESPONSE_OK:
            if not login or not password:
                continue
            try:
                pws.ClientLogin(login, password)
            except gdata.service.BadAuthentication: 
                message = _("Wrong username/password, try again")
                continue

            auth_token = gnomekeyring.item_create_sync(
                keyring,
                gnomekeyring.ITEM_GENERIC_SECRET,
                "IntiPunku Picasaweb Login, login information",
                dict(appname="IntiPunky, keyring"),
                "\n".join((login, password)), True)
            config["picasa_token"] = auth_token
            config.save()
            return login, password
        elif response == gtk.RESPONSE_CANCEL:
            return None, None

class get_facebook_login:
    def __init__(self, fb, profiled):
        self.fb = fb
        url = fb.get_login_url(popup=True)
        gtkmozembed.set_profile_path(profiled, "intisurf")
        self.moz = gtkmozembed.MozEmbed()
        
                
        win = gtk.Window()
        win.set_icon(utils.load_icon("intipunku",size=48))
        win.set_title(_("Please log in to Facebook"))
        win.set_size_request(650,500)
        win.add(self.moz)
        self.moz.load_url(url)
        self.moz.connect("open-uri", self.event)
        win.show_all()
        self.win = win
        self.loop = gobject.MainLoop()
        self.loop.run()

    def event(self, moz, url):
        
        if url == "https://login.facebook.com/login.php?popup=1&canvas=1":
            gobject.timeout_add(3000, self.get_session)

    def get_session(self):
        try:
            session = self.fb.auth.getSession()
            uid = session['uid']
            self.loop.quit()
            self.win.destroy()
            
            
        except facebook.FacebookError, e:
            print e    

class SunHTML:
    def __init__(self, photos, template="white", title="IntiPunku Photo Page",logo=DATADIR+os.sep+"logo.png", size=(640,480)):



        # Bring up the dialog to get user preferences
        dialog = dialogs.HTMLExportDialog(DATADIR + os.sep + "templates_web", title)
        r = dialog.run()
        
        if r == gtk.RESPONSE_OK:
            title, folder, template, size = dialog.get_data()
            dialog.destroy()
        else:
            dialog.destroy()
            return
        self.webfolder = folder + os.sep + title
        progress = utils.SunProgress(_("Generating HTML page"))
        n_actions = len(photos)*2 + 2
        progress.start(n_actions)

        n = 2
        webfolder = self.webfolder
        while os.path.exists(self.webfolder):
            self.webfolder = webfolder + str(n)
            n += 1

        dir_util.mkpath(self.webfolder + os.sep + "images")
        dir_util.mkpath(self.webfolder + os.sep + "thumbs")
        dir_util.mkpath(self.webfolder + os.sep + "targets")
        
        progress.set_text(_("Creating logos")) # Progress
        tools.resize_image(logo, self.webfolder + os.sep + "logo.png", (100,100), type="PNG")
        progress.plus() # Progress
        tools.resize_image(logo, self.webfolder + os.sep + "logo_small.png", (25,25), type="PNG")
        progress.plus() # Progress

        f = open(DATADIR + os.sep + "templates_web" + os.sep + template + ".html")
        html = f.read()
        f.close()
        html = "<meta http-equiv=Content-Type content=\"text/html; charset=UTF\">\n"+html
        html = html.replace("%title", unicode(title))

        html2 = ""
        for photo in photos:
            basename = os.path.basename(photo)
            progress.set_text(_("Processing %s") % basename) # Progress
            image_path = self.webfolder + os.sep + "images" + os.sep + basename
            thumb_path = self.webfolder + os.sep + "thumbs" + os.sep + basename
            tools.resize_image(photo, image_path, size)   
            progress.plus() # Progress
            tools.resize_image(photo, thumb_path, (200,200))
            progress.plus() # Progress
            html2 += "<a href=\"targets/%s.html\"><img border=\"0\" src=\"thumbs/%s\"></a>\n" % (basename, basename)

            # Create first,prev,next, last button on target page
            index = "<a href=\"../index.html\">%s</a>" % (_("Index"))
            first = "<a href=\"../targets/%s.html\">%s</a>" % (os.path.basename(photos[0]),_("First"))
            last = "<a href=\"../targets/%s.html\">%s</a>" % (os.path.basename(photos[-1]),_("Last"))
            if photos[-1] == photo:
                next = _("Next")
            else:
                next = "<a href=\"../targets/%s.html\">%s</a>" % (os.path.basename(photos[photos.index(photo)+1]),_("Next"))
            if photos[0] == photo:
                previous = _("Previous")
            else:
                previous = "<a href=\"../targets/%s.html\">%s</a>" % (os.path.basename(photos[photos.index(photo)-1]),_("Previous"))
            header = "%s|%s|%s|%s|%s" % (index, first, previous, next, last)
            html_t =html.replace("%navigate", header)
            self.create_target(image_path, html_t)
        html = html.replace("%navigate", "")
        html = html.replace("%pictures", html2)
        html = html.replace("%logo", "logo.png")
        html = html.replace("%lsmall", "logo_small.png")
        f = open(self.webfolder + os.sep + "index.html", "w")
        f.write(html)
        f.close()
        progress.destroy()


    def create_target(self, photo, html):
        html = html.replace("%logo", "../logo.png")
        html = html.replace("%lsmall", "../logo_small.png")
        target_html = html.replace("%pictures", "<img src=\"../images/%s\">" % os.path.basename(photo))
        f = open(self.webfolder + os.sep + "targets" + os.sep + os.path.basename(photo) + ".html", "w")
        f.write(target_html)
        f.close()



class PicasaWeb:
    def __init__(self, files, calbum, config):
        username, password = get_picasa_login(config)
        if username == None:
            return
        
        progress = dialogs.SunProgress(_("Uploading photos"), cancel=True, icon="picasaweb")
        progress.start(1, _("Logging in to Google"))
        pws = gdata.photos.service.PhotosService()
        utils.sunthread(pws.ClientLogin,(username, password))
        progress.plus()

        #Get all albums
        userfeed_albums = pws.GetUserFeed(kind='album')
        if progress.cancel: return

        albumsi = {} # to get the xml feed later
        albums = [] # to have everying in an order


        # Add the other albums
        n = 1
        for a in userfeed_albums.entry:
            albumsi[n] = a
            albums.append(a.title.text)
            n += 1

        # Prepare the current album - possible not created - in the list of albums, so we can process it later
        if not unicode(calbum) in albums:
            albums.insert(0, calbum)
            albumsi[0] = None
            i = 0
        else:
            i = albums.index(calbum)

        dialog = dialogs.PicasaWebDialog(albums, i)
        r = dialog.run()
        if r == gtk.RESPONSE_CANCEL:
            dialog.destroy()
            progress.destroy()
            return     

        # Handle album stuff
        i, album = dialog.get_album()
        if i == -1:
            albumf = None
        else:
            if not albumsi.has_key(0):
                i += 1
            albumf = albumsi[i]
       
        if albumf == None:
            albumf = pws.InsertAlbum(title=album, summary="")

        # Handle Image size stuff
        size = dialog.get_size()
        if size != _("Original"): size = size.split("x")
        tmpd = tempfile.mkdtemp(prefix="intipunku-")
            
        dialog.destroy()
        progress.start(len(files), text=_("Uploading photos"))
        for f in files:
            if progress.cancel:
                break
            if size != _("Original"):
                tmpf = tmpd + os.sep + os.path.basename(f)
                im = Image.open(f)
                im.thumbnail(size)
                im.save(tmpf)
                f = tmpf
            progress.set_text(_("Uploading %s" % os.path.basename(f)))
            utils.sunthread(pws.InsertPhotoSimple, (albumf, os.path.basename(f), "",f))
            progress.plus()

        # clean up
        dir_util.remove_tree(tmpd)

        progress.destroy()        

class Flickr:
    def __init__(self, files, calbum, public=1, friend=1, family=1):
        flickr = flickrapi.FlickrAPI(FLICKR_API_KEY, FLICKR_API_SECRET)
        (token, frob) = flickr.get_token_part_one(perms='write')
        if not token:
            dialog = gtk.MessageDialog(flags=gtk.DIALOG_MODAL, type=gtk.MESSAGE_WARNING, buttons=gtk.BUTTONS_OK)
            dialog.set_markup(_("A webbrowser window has been opened. Please click OK when you have allowed intipunku to acces flickr."))
            dialog.set_title(_("Please wait..."))
            r = dialog.run()
            dialog.destroy()  
        flickr.get_token_part_two((token, frob))  
        progress = dialogs.SunProgress(_("Uploading photos"), cancel=True, icon="flickr")
        progress.start(len(files))
        for f in files:
            if progress.cancel: break
            flickr.upload(filename=f, callback=self.progress, tags='''"%s"''' % (calbum), is_public=public, is_friend=friend, is_family=family)
            progress.plus()
        progress.destroy()
        
    def progress(self, progress, done):
        if progress:
            while gtk.events_pending(): gtk.main_iteration() 
            
