#!/usr/bin/env python
#
#   This file is part of IntiPunku
#   Copyright (C) 2007 Mark Baas <mark.baas123@gmail.com>
#
#   This program is free software: you can redistribute it and/or modify
#   it under the terms of the GNU Affero General Public License as
#   published by the Free Software Foundation, either version 3 of the
#   License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU Affero General Public License for more details.
#
#   You should have received a copy of the GNU Affero General Public License
#   along with this program.  If not, see <http://www.gnu.org/licenses/>.

from threading import Thread
import gtk, gobject, time
gobject.threads_init()
import os, sys, os.path

# Localization
import gettext
APP = 'intipunku'
if os.path.exists("po"):
    gettext.bindtextdomain(APP, "po")
elif os.environ.has_key("LOCALEDIR"):
    gettext.bindtextdomain(APP, os.environ["LOCALEDIR"])

gettext.textdomain(APP)
_ = gettext.gettext

try:
    f = __file__
except:
    __file__ = sys.executable
__dir__ = os.path.realpath(os.path.dirname(__file__))
sys.path.append(__dir__)
DATADIR = __dir__ + os.sep + "data"
os.environ["INTIDATA"] = DATADIR

from inti.constants import *

if WIN:
    os.environ["PATH"] += VSEP + (__dir__ + os.sep + "bin")

from inti import utils
from inti import watcher
from inti import config
from inti.editor import SunEditor
from inti.browser import SunBrowser, SunMiniBrowser
from inti.slide import Slideshow
from inti.web import SunHTML, PicasaWeb, Flickr
from inti import web
SunHTML = web.SunHTML
PicasaWeb = web.PicasaWeb
from inti import dialogs
from inti import tools
from inti import intiimport
from inti.tray import SunTray
from inti import intiprint
from inti import optional
from inti import metadata
from inti import swfexport
from distutils import dir_util, file_util

if MAC:
    from igemacintegration import *
    import gio,atk,pangocairo

__title__ = "IntiPunku"
__version__ = str(VERSION)
f = open(__dir__ + os.sep + "credits.txt")
__credits__ = f.read()
__authors__ = __credits__.split("Translators:")[0].strip()
__translators__ = __credits__.split("Translators:")[1].strip()
f.close()

class IntiPunku:
    def __init__(self):

        """
        All functions related to checking the environment should be put here
        """


        """
        Load configuration
        """

        if "--config" in sys.argv:
            self.confdir = sys.argv[sys.argv.index("--config")+1]
        else:
            self.confdir = CONFDIR
        self.cachedir = self.confdir + os.sep + "cache"

        self.config = config.SunConfig(self.confdir)
        if self.config["first_time"]:
            dialog = config.DialogFirst(self.config)
            r = dialog.run()
            dialog.destroy()      
            if r == gtk.RESPONSE_CANCEL:
                gobject.idle_add(gtk.main_quit)
                return
        self.config.init()
        # All fine, initialize the program
        self.init()         

    def init(self):

        """
        Proceed configuration
        """

        if self.config["version"] == 0.1: # Rebuild cache
            try: dir_util.remove_tree(self.cachedir)
            except: pass
        self.config["version"] = __version__
        self.config.save()  

        """
        Set up watcher to watch activity in folder
        """
        
        self.albumdir = self.config["photo_folder"]
        self.watcher = watcher.SunWatcher(self.albumdir, self.cachedir, __version__)
        self.imindex = self.watcher.imindex
        self.dindex = self.watcher.dindex
        self.tagindex = self.watcher.tagindex
        self.alltags = []
        for im in self.tagindex:
            for tag in self.tagindex[im]:
                if tag.strip() == "":
                    continue
                if not tag in self.alltags:
                    self.alltags.append(tag)
	
        
        """
        Load main window
        """

        self.win = gtk.Window()
        self.win.set_default_size(600,400)
        self.win.maximize()
        self.win.set_icon_from_file(DATADIR + os.sep + "logo-48x48.png")
        self.win.connect("delete-event", self.sunset)
        self.win.set_title(_("IntiPunku - The Sun Gate Photo Manager"))
        self.win.show()


        """
        Setting up other widgets
        """

        self.notebook = gtk.Notebook()
        self.notebook.set_show_tabs(False) 

        self.phototray = SunTray()
        self.imbrowser = SunBrowser(self.imbrowser_cb, self.imindex, self.dindex, self.tagindex, self.win, self.config)

        """
        Menu Bar
        """
        menubar = self.create_menus()            

        mvbox = gtk.VBox()
        mvbox.pack_start(menubar, expand=False, fill=False)
        mvbox.pack_start(self.notebook, expand=True, fill=True)

        self.win.add(mvbox) 

        """
        Control panel
        """
        hbox = gtk.HBox()
        
        label = gtk.Label(str=_("Photo Tray"))
        vbox2 = gtk.VBox()
        vbox2.pack_start(label, fill=False, expand=False)        
        vbox2.pack_start(self.phototray, fill=False, expand=False)

        hbox.pack_start(vbox2, fill=False, expand=False)
        
        #toolbar
        vboxt = gtk.VBox()
        vboxt.set_border_width(8)
        toolbar = self.uimanager.get_widget("/Toolbar")
        toolbar.set_style(gtk.TOOLBAR_BOTH)
        vboxt.pack_start(toolbar, fill=False, expand=False)
        hbox.pack_start(vboxt)

        vbox = gtk.VBox()
        vbox.pack_start(self.imbrowser, fill=True, expand=True)
        vbox.pack_end(hbox, fill=False, expand=False)
        vbox.set_spacing(5)
        vbox.set_border_width(10)

        self.minibrowser = SunMiniBrowser(self.imindex, callback=self.imbrowser_cb)
        self.editor = SunEditor(self.win, self.edit_cb, minibrowser=self.minibrowser)
        self.editor.connect("image-changed", self.general_cb, "image-changed")
        self.minibrowser.init(self.editor, self.imbrowser)
        self.minibrowser.album_changed_cb(None, self.imbrowser.viewer.box_load)
        self.notebook.append_page(vbox)
        self.notebook.append_page(self.editor)           

        self.win.show_all()
        if MAC: menubar.hide() #The menubar should be kept hidden
        
        self.notebook.set_current_page(0)      

        # Monitor files on change
        Thread(target=self.watcher.watch_thread, args=(self.watcher_cb,)).start()
        #gobject.timeout_add(10000, self.watcher.watch, self.watcher_cb)
       
    def sunset(self, *args):
        self.watcher.watcher_stop = True
        gtk.main_quit()

    def general_cb(self, *args):
        if args[-1] == "image-changed":
            images = list(args[1:-1])
            self.imbrowser.set_selection(images)

    def imbrowser_cb(self, *args):
        action = args[0]
        if action == "button3":
            #gobject.timeout_add(100, self.imbrowser_menu)
            self.imbrowser_menu()
        if action == "phototray":
            action2 = args[1]
            if action2 == "add":
                self.phototray.add(args[2])
            elif action2 == "remove":
                self.phototray.remove(args[2])
            else:
                self.phototray.clear()
        if action == "album_popup":
            menu = self.uimanager.get_widget("/Album")
            menu.popup(None,None,None,3,0)

        if action == "rename_album":
            self.rename_album(args[1])

        if action == "edit":
            self.switch_edit(args[1])
        if action == "savedraw":
            self.editor.draw.save_to_file()
        if action == "reload_tags":
            self.reload_tags()
        if action == "slideshow":
            win = Slideshow(self.imbrowser.get_selection())
            win.show_all()
        if action == "album-changed":
        	self.imbrowser.emit("album-changed", self.imbrowser.viewer.box_load)
        


    def edit_cb(self, *args):
        action = args[0]
        if action == "update-thumbs":
            print "This function is depreciated"
            self.imbrowser.viewer.update_thumb(args[1])
            self.imbrowser.viewer.update_date(args[1])
            self.minibrowser.update_thumb(args[1])
        if action == "sunset_editor":
            self.notebook.set_current_page(0)
        if action == "datetags":
            im = self.editor.imf
            date = self.dindex[im]
            tags = self.tagindex[im]
            return date,tags
        if action == "thumb":
            im = self.editor.imf
            return self.imindex[im]
            
            
    def watcher_cb(self, action, flist):
        if action == "update":
            for f in flist:
                self.imbrowser.viewer.update_thumb(f)
                self.imbrowser.viewer.update_date(f)
                self.minibrowser.update_thumb(f)
        elif action == "add":
            self.imbrowser.refresh_albums(select=self.imbrowser.album)
            sort_alb = False
            for f in flist:
                alb = os.path.basename(os.path.dirname(f))
                if alb == self.imbrowser.album:
                    sort_alb = True
                    self.imbrowser.viewer.add_thumb_simple(f)
            if sort_alb:
                self.imbrowser.viewer.sort_thumbs()
        elif action == "del":
            self.imbrowser.refresh_albums(select=self.imbrowser.album)
            sort_alb = False
            for f in flist:
                alb = os.path.basename(os.path.dirname(f))
                if alb == self.imbrowser.album:
                    sort_alb = True
                    self.imbrowser.viewer.remove_thumb_simple(f)
            if sort_alb:
                self.imbrowser.viewer.sort_thumbs()
        elif action == "notify":
            f = flist[0]
            msg = flist[1]
            utils.notify(f, msg)

    def rename_album(self, new):
        path1 = self.albumdir + os.sep + self.imbrowser.album
        path2 = self.albumdir + os.sep + new
        if os.path.exists(path2):
            utils.report_error(_("Cannot change album name: There is already an album with this name"))
            return
        # Rename dir
        os.rename(path1, path2)
        # Change imindex
        todelete = []
        for im in list(self.imindex):
            c = os.path.basename(os.path.dirname(im))
            if c == self.imbrowser.album:
                basename = os.path.basename(im)
                nim = os.path.abspath(path2 + os.sep + basename)
                self.imindex[nim] = self.imindex[im]
                self.dindex[nim] = self.dindex[im]
                self.tagindex[nim] = self.tagindex[im]
                todelete.append(im)
                del self.imindex[im]
                del self.dindex[im]
                del self.tagindex[im]

        self.watcher.rename_folder(path1, path2)
        


    #########################
    # Popup menu ImBrowser
    # 

    def imbrowser_menu(self):      
        popup = self.uimanager.get_widget("/Pictures")     
        popup.popup(None,None,None,3,0)                    

    #
    #########################

    #########################
    # Setup menus


    def create_menus(self):
        utils.register_icons()
        self.uimanager = gtk.UIManager()
        accelgroup = self.uimanager.get_accel_group()
        self.win.add_accel_group(accelgroup)

        f = open(DATADIR + os.sep + "menus.xml")
        xml = f.read()
        f.close()
        xmltags = ""
        actionstags = []
        for tag in self.alltags:
            xmltags += "<menuitem action=\"%s\"/>\n" % tag
            actionstags.append((tag, None, tag, None, None, self.handle_tags, False))
        xml = xml.replace("<menuitem action=\"alltags\"/>", xmltags.strip())

        # Mac menu adjustment
        # Eliminate the main intipunku main
        if MAC:
            b = xml.find("<menu action=\"intipunku\">")
            e = xml[b:].find("</menu>")+b+len("</menu>")
            xml = xml[:b]+xml[e:]
            xml = xml.replace("<menu action=\"actions\">","<menu action=\"actions\">\n<menuitem action=\"settings\"/>\n<menuitem action=\"quit\"/>\n")

        actiongroup = gtk.ActionGroup('menuactions')
        actions = [("intipunku", None, "Intipunku", None, None, None),
                 ("settings", gtk.STOCK_PREFERENCES, _("_Preferences"), None, None, self.handle_item),
                 ("quit", gtk.STOCK_QUIT, _("E_xit"), None, None, self.handle_item),
                 ("help", None, _("_Help"), None, None, None),
                 ("about", gtk.STOCK_ABOUT, None, None, None, self.handle_item),
                 ("rename_album", None, _("Rename album"), None, None, self.handle_item),
                 ("edit", "gtk-edit", _("_Edit mode"), None, None, self.handle_item_gt0),
                 ("wall", "wallpaper", _("Set as wallpaper"), None, None, self.handle_item_gt0),
                 ("export", "export", _("Export photos"), None, None, self.handle_item_gt0),
                 ("actions", None, _("_Edit"), None, None, None),
                 ("selectall", "gtk-select-all", _("Select all"), "<control>a", None, self.handle_item),
                 ("rleft", "rotate-left", _("Rotate left"), "<control>Left", None, self.handle_item_gt0),
                 ("rright", "rotate-right", _("Rotate right"),"<control>Right", None, self.handle_item_gt0),
                 ("tags", "tag2", _("Add/remove tags"), None, None, self.handle_item),
                 ("newtag", None, _("New tag"), None, None, self.handle_item_gt0),
                 ("removetags", None, _("Remove all tags"), None, None, self.handle_item_gt0),
                 ("date", "clock", _("Change time/date"), None, None, self.handle_item_gt0),
                 ("slideshow", "slideshow", _("Slideshow"), None, None, self.handle_item),
                 ("pano", "panorama", _("Create _panorama"), None, None, self.handle_item_gt1),
                 ("tools", None, _("_Tools"), None, None, None),
                 ("import", "import", _("Import photos"), None, None, self.handle_item),
                 ("removedisk", gtk.STOCK_DELETE, _("Remove from disk"), None, None, self.handle_item_gt0),
                 ("print", "gtk-print", _("Print"), None, None, self.handle_item_gt0),
                 ("slideswf", "flash", _("Flash slideshow"), None, None, self.handle_item_gt1),
                ]
        actiongroup.add_actions(actions)
        actiongroup.add_toggle_actions(actionstags)


        self.uimanager.insert_action_group(actiongroup, 0)
        self.uimanager.add_ui_from_string(xml)

        if MAC:
            macmenu = MacMenu()
            macmenu.set_menu_bar(self.uimanager.get_widget('/MenuBar'))
            quit_item = self.uimanager.get_widget('/MenuBar/actions/quit')
            macmenu.set_quit_menu_item(quit_item)
            group = macmenu.add_app_menu_group()
            group.add_app_menu_item(self.uimanager.get_widget('/MenuBar/actions/settings'), None)
            macdock = MacDock()
            macdock.connect('quit-activate', lambda d: self.sunset())

        return self.uimanager.get_widget('/MenuBar')

    def handle_item_gt0(self, widget):
        action = widget.get_name()

        if not len(self.imbrowser.get_selection()) > 0:
            utils.report_error(_("Your selection should exist of one or more photos"), "warning")
            return 

        if action == "date":
            dialog = dialogs.DateDialog(self.imbrowser.get_selection(), self.dindex)
            r = dialog.run()
            dialog.destroy()
            if r == -3:
                dt = dialog.get_datetime()
                progress = dialogs.SunProgress(_("Changing time/date"))
                progress.start(len(self.imbrowser.get_selection()))
                for path in self.imbrowser.get_selection():
                    progress.set_text(os.path.basename(path))
                    self.watcher.set_date(path, dt)
                    self.imbrowser.viewer.update_date(path)
                    self.editor.update_label()
                    progress.plus()
                progress.destroy()
                self.imbrowser.refresh_albums(select=self.imbrowser.album)

        if action == "removetags":
            dialog = gtk.MessageDialog(type=gtk.MESSAGE_QUESTION, buttons=gtk.BUTTONS_OK_CANCEL)
            dialog.set_icon_name("dialog-question")
            dialog.set_markup(_("Are you sure you want to remove all tags from the selected file(s)"))
            r = dialog.run()
            self.watcher.hold_thread = True
            if r == -5:
                progress = utils.SunProgress(_("Removing tags"))
                progress.start(len(self.imbrowser.get_selection()))
                for im in self.imbrowser.get_selection():
                    self.tagindex[im] = []
                    metadata.write_tags(im, self.tagindex[im])
                    progress.plus()
                progress.destroy()
                self.create_menus()
            dialog.destroy()
            #self.imbrowser.viewer.update(self.imbrowser.album, vt=self.imbrowser.viewtype)
            
            self.watcher.hold_thread = False

        if action == "newtag":
            dialog = gtk.MessageDialog(type=gtk.MESSAGE_QUESTION, buttons=gtk.BUTTONS_OK)
            dialog.set_icon_name("dialog-question")
            dialog.set_markup(_("Please enter a name:"))
            entry = gtk.Entry()
            entry.show()
            dialog.vbox.pack_start(entry, fill=False, expand=False)
            r = dialog.run()
            if r == -5:
                progress = utils.SunProgress(_("Adding tags"))
                progress.start(len(self.imbrowser.get_selection()))
                for im in self.imbrowser.get_selection():
                    tag = entry.get_text()
                    self.tagindex[im].append(tag)
                    metadata.write_tags(im, self.tagindex[im])
                    self.alltags.append(tag)
                    progress.plus()
                progress.destroy()
                self.create_menus()
                    
            dialog.destroy()
            #self.imbrowser.viewer.update(self.imbrowser.album, vt=self.imbrowser.viewtype)
                   

        if action == "edit":
            self.switch_edit(self.imbrowser.get_selection()[0])
        if action == "wall":
            tools.set_wallpaper(self.imbrowser.get_selection()[0])
        if action == "html":
            SunHTML(self.imbrowser.get_selection(), self.config["web_exports"], title=self.imbrowser.viewer.title)
        if action == "picasaweb":
            # gdata is a optional component check for existence
            if not optional.check("gdata", web.load_gdata, web.GDATA_ENABLED): return    
            PicasaWeb(self.imbrowser.get_selection(),self.imbrowser.album, self.config)
        if action == "flickr":
            # flickr is a optional component check for existence
            if not optional.check("flickr", web.load_flickr, web.FLICKR_ENABLED): return  
            
            dialog = dialogs.FlickrDialog(self.imbrowser.album)
            r = dialog.run()

            if r == gtk.RESPONSE_CANCEL:
                dialog.destroy(); return
            data = dialog.get_data()
            dialog.destroy()
            
            Flickr(self.imbrowser.get_selection(), *data)
            
        if action == "slideswf":
            # should be gt1, so redirecting
            self.handle_item_gt1(widget)
         
       
        if action == "rleft":
            for im in self.imbrowser.get_selection():
                self.editor.draw.load(im)
                self.editor.draw.rotate_left()
        if action == "rright":
            for im in self.imbrowser.get_selection():
                self.editor.draw.load(im)
                self.editor.draw.rotate_right()

        if action == "export":
            dialog = dialogs.ChooseExportDialog(self.handle_item_gt0)
            r = dialog.run()
            w = dialog.get_export()
            dialog.destroy()
            if r == gtk.RESPONSE_OK:
                self.handle_item_gt0(w)

        if action == "removedisk":
            f = self.imbrowser.get_selection()[0]

            dialog = gtk.MessageDialog(type=gtk.MESSAGE_WARNING, buttons=gtk.BUTTONS_OK_CANCEL)
            dialog.set_icon_name("dialog-question")
            dialog.set_markup(_("Are you sure you want to delete this photo?"))
            r = dialog.run() 
            if r == gtk.RESPONSE_OK:
                try: os.remove(os.path.dirname(f) + os.sep + "Originals" + os.sep + os.path.basename(f))
                except: pass
                try: os.remove(f)
                except: pass
                
            dialog.destroy()
            
        if action == "print":
            dialog = dialogs.PrintDialog(title=self.imbrowser.viewer.title)
            r = dialog.run()
            title, n = dialog.get_data()
            settings = dialog.get_settings()
            dialog.destroy()
            if r == gtk.RESPONSE_OK:
                toprint = []
                for f in self.imbrowser.get_selection():
                    toprint.append((self.dindex[f],f))
                intiprint.intiprint(toprint, title, layout=PRINT_LAYOUT[n], settings=settings, win=self.win, filename=self.config["print_filename"], date=self.config["print_date"])   	
            

    def handle_item_gt1(self, widget):
        action = widget.get_name()
        
        if not len(self.imbrowser.get_selection()) > 1:
            utils.report_error(_("Your selection should exist of two or more photos"), "warning")
            return

        if action == "pano":
            # Check if autopano-(c), hugin-tools and is installed
            if not optional.check("pano", tools.load_pano, tools.PANO_ENABLED): return 
            dialog = gtk.MessageDialog(flags=gtk.DIALOG_MODAL, type=gtk.MESSAGE_WARNING, buttons=gtk.BUTTONS_OK_CANCEL)
            dialog.set_markup(_("Intipunki is about to start panorama creation.\nCreating panoramas is quite a time consuming process, \nare you sure you want to continue?"))
            dialog.set_title(_("Intipunku panorama creation"))
            r = dialog.run()
            dialog.destroy()  
            if r == gtk.RESPONSE_CANCEL:
                return
            
            pano = tools.pano(self.imbrowser.get_selection())
            

        if action == "slideswf":
            # Ming is optional check
            if not optional.check("ming", swfexport.load_ming, swfexport.MING_ENABLED): return 
            dialog = dialogs.FlashExportDialog(album=self.imbrowser.album)
            r = dialog.run()       
            folder, title = dialog.get_output_info() 
            dialog.destroy()     
            if r == -5:
                swfexport.create_swfslide(self.imbrowser.get_selection(), folder + os.sep + title.replace(" ", "_").lower() + ".swf")
            

    def handle_item(self, widget):
        action = widget.get_name()

        if action == "tags":
            self.load_tagmenu()

        if action == "settings":
            dialog = dialogs.IntiSettings(self.config)
            r = dialog.run()
            if r == gtk.RESPONSE_ACCEPT:
                dialog.save_settings()
                if self.config["default_import"]:
                    utils.set_default_import()
            dialog.destroy()

        if action == "quit":
            self.sunset()

        if action == "about":
            dialog = dialogs.AboutDialog(__title__, __version__, __authors__, __translators__, __dir__ + os.sep + "license.txt")
            dialog.run()
            dialog.destroy()
           
        if action == "rename_album":
            self.imbrowser.set_editing()
        if action == "selectall":
            self.imbrowser.select_all()
    
        if action == "import":
            dialog = dialogs.ChooseImportDialog()
            r = dialog.run()
            dialog.destroy()
            if r >= 10 and r <= 12:
                t, l = page_labels[r-10]
                # gdata is a optional component check for existence
                if t == "picasaweb":
                    if not optional.check("gdata", intiimport.load_gdata, intiimport.GDATA_ENABLED): return   
                # gphoto is a optional component check for existence
                if t == "camera":
                    if not optional.check("gphoto", intiimport.load_gphoto, intiimport.GPHOTO_ENABLED): return   
                albums = self.imbrowser.collections
                dialog2 = dialogs.ImportDialog()
                page = intiimport.pages[t](dialog2, albums, self.config, callback=self.handle_import) 
                
                
                dialog2.set_page(page)
                r = dialog2.run()
                if r == gtk.RESPONSE_OK:
                    page.doImport()
                dialog2.destroy()

        if action == "slideshow":
            win = Slideshow(self.imbrowser.get_selection(select_all=True), self.config)
            win.show_all()
     

    def handle_import(self, name, *args):
        if name == "sunset":
            gobject.idle_add(args[0].destroy) 

    def handle_tags(self, actionw):
        toggled = actionw.get_active()
        name = actionw.get_name()
        if self.handle_toggle_tag == False: return
        if toggled:
            for im in self.imbrowser.get_selection():
                if not name in self.tagindex[im]:
                    self.tagindex[im].append(name)
        elif toggled == False:
            for im in self.imbrowser.get_selection():
                if name in self.tagindex[im]:
                    self.tagindex[im].remove(name)
        progress = dialogs.SunProgress(_("Adding tags"))
        progress.start(len(self.imbrowser.get_selection()))
        for im in self.imbrowser.get_selection():
            metadata.write_tags(im, self.tagindex[im])
            progress.plus()
        progress.destroy()
        #self.imbrowser.viewer.update(self.imbrowser.album, vt=self.imbrowser.viewtype)
        
        self.create_menus()

    def load_tagmenu(self):
        tagsd = {}
        for tag in self.alltags:
            tagsd[tag] = True
            for im in self.imbrowser.get_selection():
                if not tag in self.tagindex[im]:
                    tagsd[tag] = False
        tags = []        
        for tag in tagsd:
            if tagsd[tag] == True:
                tags.append(tag)
        tags.sort()

        menu = self.uimanager.get_widget("/Pictures/Tags").get_submenu()
        for i in menu.get_children():
            if type(i) == type(gtk.CheckMenuItem()):
                self.handle_toggle_tag = False
                if i.get_name() in tags:
                    i.set_active(True)
                else:  
                    i.set_active(False)
                self.handle_toggle_tag = True

    def reload_tags(self):
        self.tagindex = metadata.reload_tags(self.tagindex)
        self.alltags = []
        for im in self.tagindex:
            for tag in self.tagindex[im]:
                if not tag in self.alltags:
                    self.alltags.append(tag)
        self.update_index_childs()

    def switch_edit(self, im):
        self.editor.load(im)
        self.notebook.set_current_page(1)
        

                


if __name__ == "__main__":
    try:
        import psyco
        psyco.profile()
    except:
        if not MAC: print "The psyco module does not seem to be installed. It is not necessary, however it can speed up performance."


    app  = IntiPunku()

    try: gtk.main()
    except KeyboardInterrupt, err:
        print err
        sys.exit()
